/*
 * Copyright LWJGL. All rights reserved.
 * License terms: https://www.lwjgl.org/license
 * MACHINE GENERATED FILE, DO NOT EDIT
 */
package org.lwjgl.llvm;

import javax.annotation.*;

import java.nio.*;

import org.lwjgl.*;
import org.lwjgl.system.*;

import static org.lwjgl.system.Checks.*;
import static org.lwjgl.system.MemoryUtil.*;
import static org.lwjgl.system.MemoryStack.*;

/**
 * Uniquely identifies a CXFile, that refers to the same underlying file, across an indexing session.
 * 
 * <h3>Layout</h3>
 * 
 * <pre><code>
 * struct CXFileUniqueID {
 *     unsigned long long data[3];
 * }</code></pre>
 */
public class CXFileUniqueID extends Struct implements NativeResource {

    /** The struct size in bytes. */
    public static final int SIZEOF;

    /** The struct alignment in bytes. */
    public static final int ALIGNOF;

    /** The struct member offsets. */
    public static final int
        DATA;

    static {
        Layout layout = __struct(
            __array(8, 3)
        );

        SIZEOF = layout.getSize();
        ALIGNOF = layout.getAlignment();

        DATA = layout.offsetof(0);
    }

    /**
     * Creates a {@link CXFileUniqueID} instance at the current position of the specified {@link ByteBuffer} container. Changes to the buffer's content will be
     * visible to the struct instance and vice versa.
     *
     * <p>The created instance holds a strong reference to the container object.</p>
     */
    public CXFileUniqueID(ByteBuffer container) {
        super(memAddress(container), __checkContainer(container, SIZEOF));
    }

    @Override
    public int sizeof() { return SIZEOF; }

    /** Returns a {@link LongBuffer} view of the {@code data} field. */
    @NativeType("unsigned long long[3]")
    public LongBuffer data() { return ndata(address()); }
    /** Returns the value at the specified index of the {@code data} field. */
    @NativeType("unsigned long long")
    public long data(int index) { return ndata(address(), index); }

    // -----------------------------------

    /** Returns a new {@link CXFileUniqueID} instance allocated with {@link MemoryUtil#memAlloc memAlloc}. The instance must be explicitly freed. */
    public static CXFileUniqueID malloc() {
        return wrap(CXFileUniqueID.class, nmemAllocChecked(SIZEOF));
    }

    /** Returns a new {@link CXFileUniqueID} instance allocated with {@link MemoryUtil#memCalloc memCalloc}. The instance must be explicitly freed. */
    public static CXFileUniqueID calloc() {
        return wrap(CXFileUniqueID.class, nmemCallocChecked(1, SIZEOF));
    }

    /** Returns a new {@link CXFileUniqueID} instance allocated with {@link BufferUtils}. */
    public static CXFileUniqueID create() {
        ByteBuffer container = BufferUtils.createByteBuffer(SIZEOF);
        return wrap(CXFileUniqueID.class, memAddress(container), container);
    }

    /** Returns a new {@link CXFileUniqueID} instance for the specified memory address. */
    public static CXFileUniqueID create(long address) {
        return wrap(CXFileUniqueID.class, address);
    }

    /** Like {@link #create(long) create}, but returns {@code null} if {@code address} is {@code NULL}. */
    @Nullable
    public static CXFileUniqueID createSafe(long address) {
        return address == NULL ? null : wrap(CXFileUniqueID.class, address);
    }

    /**
     * Returns a new {@link CXFileUniqueID.Buffer} instance allocated with {@link MemoryUtil#memAlloc memAlloc}. The instance must be explicitly freed.
     *
     * @param capacity the buffer capacity
     */
    public static CXFileUniqueID.Buffer malloc(int capacity) {
        return wrap(Buffer.class, nmemAllocChecked(__checkMalloc(capacity, SIZEOF)), capacity);
    }

    /**
     * Returns a new {@link CXFileUniqueID.Buffer} instance allocated with {@link MemoryUtil#memCalloc memCalloc}. The instance must be explicitly freed.
     *
     * @param capacity the buffer capacity
     */
    public static CXFileUniqueID.Buffer calloc(int capacity) {
        return wrap(Buffer.class, nmemCallocChecked(capacity, SIZEOF), capacity);
    }

    /**
     * Returns a new {@link CXFileUniqueID.Buffer} instance allocated with {@link BufferUtils}.
     *
     * @param capacity the buffer capacity
     */
    public static CXFileUniqueID.Buffer create(int capacity) {
        ByteBuffer container = __create(capacity, SIZEOF);
        return wrap(Buffer.class, memAddress(container), capacity, container);
    }

    /**
     * Create a {@link CXFileUniqueID.Buffer} instance at the specified memory.
     *
     * @param address  the memory address
     * @param capacity the buffer capacity
     */
    public static CXFileUniqueID.Buffer create(long address, int capacity) {
        return wrap(Buffer.class, address, capacity);
    }

    /** Like {@link #create(long, int) create}, but returns {@code null} if {@code address} is {@code NULL}. */
    @Nullable
    public static CXFileUniqueID.Buffer createSafe(long address, int capacity) {
        return address == NULL ? null : wrap(Buffer.class, address, capacity);
    }

    // -----------------------------------

    /** Returns a new {@link CXFileUniqueID} instance allocated on the thread-local {@link MemoryStack}. */
    public static CXFileUniqueID mallocStack() {
        return mallocStack(stackGet());
    }

    /** Returns a new {@link CXFileUniqueID} instance allocated on the thread-local {@link MemoryStack} and initializes all its bits to zero. */
    public static CXFileUniqueID callocStack() {
        return callocStack(stackGet());
    }

    /**
     * Returns a new {@link CXFileUniqueID} instance allocated on the specified {@link MemoryStack}.
     *
     * @param stack the stack from which to allocate
     */
    public static CXFileUniqueID mallocStack(MemoryStack stack) {
        return wrap(CXFileUniqueID.class, stack.nmalloc(ALIGNOF, SIZEOF));
    }

    /**
     * Returns a new {@link CXFileUniqueID} instance allocated on the specified {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param stack the stack from which to allocate
     */
    public static CXFileUniqueID callocStack(MemoryStack stack) {
        return wrap(CXFileUniqueID.class, stack.ncalloc(ALIGNOF, 1, SIZEOF));
    }

    /**
     * Returns a new {@link CXFileUniqueID.Buffer} instance allocated on the thread-local {@link MemoryStack}.
     *
     * @param capacity the buffer capacity
     */
    public static CXFileUniqueID.Buffer mallocStack(int capacity) {
        return mallocStack(capacity, stackGet());
    }

    /**
     * Returns a new {@link CXFileUniqueID.Buffer} instance allocated on the thread-local {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param capacity the buffer capacity
     */
    public static CXFileUniqueID.Buffer callocStack(int capacity) {
        return callocStack(capacity, stackGet());
    }

    /**
     * Returns a new {@link CXFileUniqueID.Buffer} instance allocated on the specified {@link MemoryStack}.
     *
     * @param stack the stack from which to allocate
     * @param capacity the buffer capacity
     */
    public static CXFileUniqueID.Buffer mallocStack(int capacity, MemoryStack stack) {
        return wrap(Buffer.class, stack.nmalloc(ALIGNOF, capacity * SIZEOF), capacity);
    }

    /**
     * Returns a new {@link CXFileUniqueID.Buffer} instance allocated on the specified {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param stack the stack from which to allocate
     * @param capacity the buffer capacity
     */
    public static CXFileUniqueID.Buffer callocStack(int capacity, MemoryStack stack) {
        return wrap(Buffer.class, stack.ncalloc(ALIGNOF, capacity, SIZEOF), capacity);
    }

    // -----------------------------------

    /** Unsafe version of {@link #data}. */
    public static LongBuffer ndata(long struct) { return memLongBuffer(struct + CXFileUniqueID.DATA, 3); }
    /** Unsafe version of {@link #data(int) data}. */
    public static long ndata(long struct, int index) {
        return UNSAFE.getLong(null, struct + CXFileUniqueID.DATA + check(index, 3) * 8);
    }

    // -----------------------------------

    /** An array of {@link CXFileUniqueID} structs. */
    public static class Buffer extends StructBuffer<CXFileUniqueID, Buffer> implements NativeResource {

        private static final CXFileUniqueID ELEMENT_FACTORY = CXFileUniqueID.create(-1L);

        /**
         * Creates a new {@link CXFileUniqueID.Buffer} instance backed by the specified container.
         *
         * Changes to the container's content will be visible to the struct buffer instance and vice versa. The two buffers' position, limit, and mark values
         * will be independent. The new buffer's position will be zero, its capacity and its limit will be the number of bytes remaining in this buffer divided
         * by {@link CXFileUniqueID#SIZEOF}, and its mark will be undefined.
         *
         * <p>The created buffer instance holds a strong reference to the container object.</p>
         */
        public Buffer(ByteBuffer container) {
            super(container, container.remaining() / SIZEOF);
        }

        public Buffer(long address, int cap) {
            super(address, null, -1, 0, cap, cap);
        }

        Buffer(long address, @Nullable ByteBuffer container, int mark, int pos, int lim, int cap) {
            super(address, container, mark, pos, lim, cap);
        }

        @Override
        protected Buffer self() {
            return this;
        }

        @Override
        protected CXFileUniqueID getElementFactory() {
            return ELEMENT_FACTORY;
        }

        /** Returns a {@link LongBuffer} view of the {@code data} field. */
        @NativeType("unsigned long long[3]")
        public LongBuffer data() { return CXFileUniqueID.ndata(address()); }
        /** Returns the value at the specified index of the {@code data} field. */
        @NativeType("unsigned long long")
        public long data(int index) { return CXFileUniqueID.ndata(address(), index); }

    }

}